@@ -33,6 +33,8 @@ validate_iterations <- function(iterations, default, name) {
 #########################
 #' @import mboost
 #' @import stats
+
+# Input should be "family`at`name", with family being a boost_family_glm object
 get_mboost_family <- function(family) {
   if (!is.character(family)) {
     stop("Family must be a character string.")
@@ -41,8 +43,8 @@ get_mboost_family <- function(family) {
   switch(family,
          # Gaussian
          "Squared Error (Regression)" = stats::gaussian(link = "identity"),
-         # Gamma
-         "Negative Gamma Likelihood" = stats::Gamma(link = "inverse"),
+         # Gamma; GammaReg():The implemented loss function is the negative Gamma log-likelihood with logarithmic link function
+         "Negative Gamma Likelihood" = stats::Gamma(link = "log"),
          # Binomial 
          "Negative Binomial Likelihood (logit link)" = stats::binomial(link = "logit"),
          "Negative Binomial Likelihood -- probit link" = stats::binomial(link = "probit"),
@@ -133,7 +135,7 @@ predict_GeDS_linear <- function(Gmod, X, Y, Z){
       }
     
     # Initialize output vector
-    Y_hat <- numeric(nrow(X))
+    F_hat <- numeric(nrow(X))
     # Number of intervals
     n_intervals <- length(knt) - 1
     # Add first and last intervals to cover all possible X values
@@ -152,11 +154,18 @@ predict_GeDS_linear <- function(Gmod, X, Y, Z){
       # check 'cross_validation\HPC\GeDS_initial_learner\ex5\additive\scripts\cv_ex5_1160.R' (seed=6967) for an example.
       b1 <- (coeff[int+1] - coeff[int]) / max(knt[int+1] - knt[int], epsilon)
       b0 <- coeff[int] - b1 * knt[int]
-      Y_hat[idx] <- b0 + b1 * X[idx]
+      F_hat[idx] <- b0 + b1 * X[idx]
       # Store coefficients
       b0_list[int] <- b0
       b1_list[int] <- b1
     }
+    
+    if (is.null(Gmod$Args$family)){
+      Y_hat <- F_hat
+    } else{
+      Y_hat <- Gmod$Args$family$linkinv(F_hat)
+    }
+    
     return(list(Y_hat = Y_hat, knt = knt, int.knt = int.knt, b0 = b0_list, b1 = b1_list, theta = coeff))
     
   ## Bivariate GeDS
@@ -186,18 +195,24 @@ predict_GeDS_linear <- function(Gmod, X, Y, Z){
     if(length(Yknt) == 2) {Yint.knt <- NULL}
     
     # SplineReg_biv
-    lin <- SplineReg_biv(X, Y, Z, InterKnotsX=Xint.knt, InterKnotsY=Yint.knt,
-                         Xextr=Gmod$Args$Xextr, Yextr=Gmod$Args$Yextr, n=2,
-                         coefficients = theta)
-    Y_hat <- lin$Predicted
+    if (Gmod$Type == "LM - Biv") {
+      lin <- SplineReg_biv(X, Y, Z, InterKnotsX = Xint.knt, InterKnotsY = Yint.knt,
+                           Xextr = Gmod$Args$Xextr, Yextr = Gmod$Args$Yextr, n = 2,
+                           coefficients = theta)
+      } else if (Gmod$Type == "GLM - Biv") {
+        lin <- SplineReg_biv_GLM(X, Y, Z, InterKnotsX = Xint.knt, InterKnotsY = Yint.knt,
+                                 Xextr = Gmod$Args$Xextr, Yextr = Gmod$Args$Yextr, n = 2,
+                                 family = Gmod$Args$family, coefficients = theta)
+        }
+    
+    Y_hat <-lin$Predicted
     
     return(list(Y_hat = Y_hat, knt = list("Xknt" = Xknt, "Yknt" = Yknt),
                 int.knt = list("Xint.knt" = Xint.knt, "Yint.knt" = Yint.knt),
                 "theta" = lin$Theta))
-    
-  } else {
-    print("Only 1 (i.e. Y ~ f(X)) or 2 (i.e. Z ~ f(X, Y)) predictors are allowed for NGeDS models.")
-  }
+    } else {
+      print("Only 1 (i.e. Y ~ f(X)) or 2 (i.e. Z ~ f(X, Y)) predictors are allowed for NGeDS models.")
+    }
 }
 
 ############################################################################
@@ -288,7 +303,8 @@ bivariate_bl_linear_model <- function(pred_vars, model, shrinkage, base_learners
     
     bl <- model$base_learners[[base_learner]]
     # (I) Bivariate boosted base learners
-    if(type == "boost"){
+    if(type == "boost") {
+      
       for (mod in names(bl$iterations)){
         Xint.knt <- bl$iterations[[mod]]$int.knt$Xint.knt
         Yint.knt <- bl$iterations[[mod]]$int.knt$Yint.knt
@@ -296,11 +312,17 @@ bivariate_bl_linear_model <- function(pred_vars, model, shrinkage, base_learners
         lin <- SplineReg_biv(X, Y, InterKnotsX=Xint.knt, InterKnotsY=Yint.knt,
                              Xextr=range(X), Yextr=range(Y), n=2,
                              coefficients = theta)
-        if (mod=="model0") {Y_hat <- Y_hat + lin$Predicted}
-        else {Y_hat <- Y_hat + shrinkage*lin$Predicted}
+        
+        if (mod=="model0") {
+          Y_hat <- Y_hat + lin$Predicted # initial learner is added with no shrinking
+        } else {
+            Y_hat <- Y_hat + shrinkage*lin$Predicted
+        }
+        
       }
     # (II) Bivariate gam
-    } else if (type == "gam"){
+    } else if (type == "gam") {
+      
       Xint.knt <- bl$linear.int.knots$ikX
       Yint.knt <- bl$linear.int.knots$ikY
       theta <- bl$coefficients
@@ -324,7 +346,7 @@ compute_avg_int.knots <- function(final_model, base_learners = base_learners, X_
   base_learners =  base_learners[sapply(base_learners, function(x) x$type == "GeDS")]
   # Check if base_learners is empty
   if (length(base_learners) == 0) {
-    cat(paste0("No GeDS base-learners found. Returning NULL when computing averaging knot location for n = ", n))
+    cat(paste0("No GeDS base-learners found. Returning NULL when computing averaging knot location for n = ", n,". "))
     return(kk_list = NULL)
   }
   
@@ -342,7 +364,7 @@ compute_avg_int.knots <- function(final_model, base_learners = base_learners, X_
       cat(paste0(bl, " has less than ", n - 1, " linear internal knots. "))
       warning_displayed <<- TRUE  # Update the warning flag
       return(intknt)
-      # Bivariate
+    # Bivariate
     } else if (length(pred_vars) == 2) {
       intknt <- list(X = get_internal_knots(final_model$base_learners[[bl]]$knots$Xk), 
                      Y = get_internal_knots(final_model$base_learners[[bl]]$knots$Yk))
@@ -372,12 +394,135 @@ compute_avg_int.knots <- function(final_model, base_learners = base_learners, X_
   # Check if any warning was displayed and show the appropriate follow-up message
   if (warning_displayed) {
     if (n == 3) {
-      cat("Quadratic averaging knot location is not computed for base learners with less than 2 linear internal knots.\n")
+      cat("Quadratic averaging knot location is not computed for base learners with less than 2 internal knots.\n")
     } else if (n == 4) {
-      cat("Cubic averaging knot location is not computed for base learners with less than 3 linear internal knots.\n")
+      cat("Cubic averaging knot location is not computed for base learners with less than 3 internal knots.\n")
     }
   }
   # Naming the elements of the list with the base-learner names
   names(kk_list) <- names(base_learners)
   return(kk_list = kk_list)
 }
+
+################################################################################
+
+bSpline.coef <- function(final_model, univariate_learners)
+  {
+  learner_names <- names(univariate_learners)
+  knots <- lapply(learner_names, function(bl) final_model$base_learners[[bl]]$knots)
+  names(knots) <- learner_names
+  poly_coef <- lapply(learner_names, function(bl) final_model$base_learners[[bl]]$coefficients)
+  names(poly_coef) <- learner_names
+  
+  epsilon <- 1e-10
+  coeff <- setNames(vector("list", length(learner_names)), learner_names) 
+  
+  # Loop through each learner
+  for (bl in learner_names) {
+    knt <- knots[[bl]]
+    b0 <- poly_coef[[bl]]$b0
+    b1 <- poly_coef[[bl]]$b1
+    
+    # Calculate coefficient
+    coeff[[bl]] <- b0 + b1 * knt[-length(knt)]
+    # Calculate the last element to append
+    last_knot_diff <- max(knt[length(knt)] - knt[length(knt) - 1], epsilon)
+    last_coeff <- coeff[[bl]][length(coeff[[bl]])] + b1[length(b1)] * last_knot_diff 
+    
+    # Append the last value to the coefficient vector
+    coeff[[bl]] <- c(coeff[[bl]], last_coeff)
+    
+    # # Check
+    # round(b1 - diff(coeff[[bl]]) / pmax(diff(knt), epsilon), 10)
+    # round(b0 - ( coeff[[bl]][-length(coeff[[bl]])] - b1 * knt[-length(knt)] ), 10)
+  }
+  
+  
+  return(coeff)
+}
+
+
+biv_bSpline.coef <- function(final_model, bivariate_learners, shrinkage)
+{
+  bivbl_names <- names(bivariate_learners)
+  biv_models <- lapply(bivbl_names, function(bl) final_model$base_learners[[bl]]$iterations)
+  names(biv_models) <- bivbl_names
+  
+  theta_biv <- setNames(vector("list", length(bivbl_names)), bivbl_names) 
+  for (bl in bivbl_names) {
+    bl_models <- biv_models[[bl]]
+    theta <- list()
+    
+    # Loop through each model in the base-learner
+    for (model_name in names(bl_models)) {
+      # Extract model number
+      model_num <- as.numeric(gsub("model", "", model_name))
+      #Extract coefs
+      coefs <- biv_models[[bl]][[model_name]]$coef
+      # Apply shrinkage
+      if (model_name != "model0") coefs <- shrinkage * coefs
+      # Store coefficients
+      theta[[paste0("theta", model_num)]] <- coefs
+      }
+    
+    # Store results for the base learner
+    theta_biv[[bl]] <- theta
+    
+  }
+  return(theta_biv)
+}
+
+
+### Validate Y_hat ##
+# At each boosting iteration, you need to transform Y_hat into F_hat
+# in order to recompute the gradient vector. Sometimes Y_hat might violate the requirements
+# of the corresponding distribution. This function ensures Y_hat meets those requirements
+validate_Y_hat <- function(Y_hat, family_stats) {
+  valid <- family_stats$validmu(Y_hat)
+  
+  if (all(valid)) {
+    return(Y_hat)
+  }
+  
+  if (family_stats$family == "binomial") {
+    # Ensure values are between 0 and 1
+    Y_hat <- pmax(pmin(Y_hat, 1), 0)
+  } else if (family_stats$family == "poisson" || family_stats$family == "Gamma" || family_stats$family == "inverse.gaussian") {
+    # Ensure values are positive
+    Y_hat <- pmax(Y_hat, .Machine$double.eps)
+  }
+  # Add more conditions for other families if necessary
+  
+  valid <- family_stats$validmu(Y_hat)
+  
+  if (!all(valid)) {
+    warning("Some Y_hat values are still invalid after correction.")
+  }
+  
+  return(Y_hat)
+}
+
+##########################
+## count_initial_zeroes ##
+##########################
+# To know the rounding when defining the shrinkage rate as 2/max(Y)
+count_initial_zeroes <- function(x) {
+  # Convert the number to a string
+  num_str <- as.character(x)
+  
+  # Split the string at the decimal point
+  parts <- strsplit(num_str, split = "\\.")[[1]]
+  
+  # Check if there's a decimal part
+  if (length(parts) < 2) {
+    return(0)  # No decimal part means no zeroes after the decimal
+  }
+  
+  # Get the decimal part
+  decimal_part <- parts[2]
+  
+  # Count the number of leading zeroes in the decimal part
+  leading_zeroes <- nchar(gsub("^0*", "", decimal_part, perl = TRUE)) - nchar(decimal_part)
+  return(-leading_zeroes + 1)
+} 
+
